@@ -5,7 +5,7 @@ package Net::CVE;
 use 5.014002;
 use warnings;
 
-our $VERSION = "0.002"; # 20230523
+our $VERSION = "0.003"; # 20230524
 
 use Carp;
 use HTTP::Tiny;
@@ -24,6 +24,7 @@ sub new {
 	ua   => undef,
 	lang => "en",
 	data => {},
+	diag => undef,
 	);
     if (@_) {
 	if (@_ == 1 && ref $_[0] eq "HASH") {
@@ -37,9 +38,26 @@ sub new {
     bless \%r => $class;
     } # new
 
+sub diag {
+    my $self = shift or return;
+    ref $self or return;
+    my $d = $self->{diag} or return;
+    unless (defined wantarray) { # void context
+	my $act = $d->{action};
+	warn "$act: ",
+	    (join " " => grep { length } $d->{status}, $d->{reason}), "\n";
+	$act =~ s/./ /g;
+	warn "$act  source = $d->{source}\n"	if $d->{source};
+	warn "$act  usage: $d->{usage}\n"	if $d->{usage};
+	}
+    return $d;
+    } # diag
+
 sub get {
     my ($self, $cve) = @_;
+    ref $self or $self = __PACKAGE__->new ();
     $self->{data} = {};
+    $self->{diag} = undef;
     $cve or return $self;
     $cve =~ s/^(?=[0-9])/CVE-/;
     if ($cve =~ m/^CVE-[0-9]{4}-([0-9]+)$/) {
@@ -47,31 +65,55 @@ sub get {
 	my $url = join "/" => $self->{url}, $cve;
 	my $r = $self->{ua}->get ($url);
 	unless ($r->{success}) {
-	    #warn "$cve: $r->{status} $r->{reason}\n";
-	    $self->{data} = {
-	     containers => {
-	      cna => {
-	       descriptions => [{
-		value => "$r->{status} $r->{reason}",
-		}],
-	       problemTypes => [{
-	        descriptions => [{
-	         description => "Fetch URL",
-	         url         => $url,
-	         type        => "Error",
-	         }],
-	       }] }}};
+	    # if pseudo-HTTP status code 599 and reason "Internal Exception"
+	    # the content field will contain the text of the error
+	    my $status = $r->{status};
+	    my $reason = join ": " => grep { length }
+		$r->{reason}, $status eq "599" ? $r->{content} : "";
+	    $self->{diag} = {
+		status => $status,
+		reason => $reason,
+		action => "get",
+		source => $url,
+		usage  => undef,
+		};
 	    return $self;
 	    }
 	$self->{data} = decode_json ($r->{content});
 	}
     elsif (-s $cve) {
-	open my $fh, "<:encoding(utf-8)", $cve or croak "$cve: $!\n";
-	$self->{data} = decode_json (do { local $/; <$fh> });
+	my $fh;
+	unless (open $fh, "<:encoding(utf-8)", $cve) {
+	    $self->{diag} = {
+		status => 0 + $!,
+		reason => "$!",
+		action => "get",
+		source => $cve,
+		usage  => 'get ("cve-2022-26928.json")',
+		};
+	    return $self;
+	    }
+	unless (eval { $self->{data} = decode_json (do { local $/; <$fh> }); 1 }) {
+	    $self->{diag} = {
+		status => -2,
+		reason => $@ =~ s{\s+at\s+\S+\s+line\s+\d+.*}{}rs,
+		action => "decode_json",
+		source => $cve,
+		usage  => undef,
+		};
+	    return $self;
+	    }
 	close $fh;
 	}
     else {
-	warn "Invalid CVE format: '$cve' - expected format CVE-2023-12345\n";
+	#warn "Invalid CVE format: '$cve' - expected format CVE-2023-12345\n";
+	$self->{diag} = {
+	    status => -1,
+	    reason => "Invalid CVE format: '$cve'",
+	    action => "get",
+	    source => "tag",
+	    usage  => 'get ("CVE-2022-26928")',
+	    };
 	return $self;
 	}
     $self;
@@ -79,12 +121,14 @@ sub get {
 
 sub data {
     my $self = shift;
+    ref $self or $self = __PACKAGE__->new ();
     @_ and $self->get (@_);
     $self->{data};
     } # data
 
 sub summary {
     my $self = shift;
+    ref $self or $self = __PACKAGE__->new ();
     @_ and $self->get (@_);
     my $j   = $self->{data}         or croak "summary only available after get";
     my $cna = $j->{containers}{cna} or return +{};
@@ -186,12 +230,14 @@ Net::CVE - Fetch CVE (Common Vulnerabilities and Exposures) information from cve
 
  my $cr = Net::CVE->new ();
 
- $cr->get ("CVE-2022-12345");
+ $cr->get ("CVE-2022-26928");
  my $full_report = $cr->data;
  my $summary     = $cr->summary;
 
+ $cr->diag;
+
  use Data::Peek;
- DDumper $cr->summary ("CVE-2022-12345");
+ DDumper $cr->summary ("CVE-2022-26928");
 
 =head1 DESCRIPTION
 
@@ -250,6 +296,15 @@ information from the internet.
 The decoded information is stored internally and will be re-used for other
 methods.
 
+C<get> returns the object and allows to omit a call to C<new> which will be
+implicit but does not allow attributes
+
+ my $reporter = Net::CVE->get ("2022-26928");
+
+is a shortcut to
+
+ my $reporter = Net::CVE->new->get ("2022-26928");
+
 =head2 data
 
  my $info = $reporter->data;
@@ -269,6 +324,10 @@ or
  $reporter->get ("CVE-2022-26928");
  my $info = $reporter->data;
 
+or even, whithout an object
+
+ my $info = Net::CVE->data ("CVE-2022-26928");
+
 =head2 summary
 
  my $info = $reporter->summary;
@@ -290,6 +349,10 @@ or
  $reporter->get ("CVE-2022-26928");
  my $info = $reporter->summary;
 
+or even, whithout an object
+
+ my $info = Net::CVE->summary ("CVE-2022-26928");
+
 The returned hash looks somewhat like this
 
  { date        => "2022-09-13T18:41:25",
@@ -353,6 +416,41 @@ Returns the list of platforms for the affected parts of the CVE. In scalar
 context a string where the (sorted) list of unique platforms is joined by
 C<, > in list context the (sorted) list itself.
 
+=head2 diag
+
+ $reporter->diag;
+ my $diag = $reporter->diag;
+
+If an error occured, returns information about the error. In void context
+prints the diagnostics using C<warn>. The diagnostics - if any - will be
+returned in a hashref with the following fields:
+
+=over 2
+
+=item status
+
+Status code
+
+=item reason
+
+Failure reason
+
+=item action
+
+Tag of where the failure occured
+
+=item source
+
+The URL or filename leading to the failure
+
+=item usage
+
+Help message
+
+=back
+
+Only the C<action> field is guaranteed to be set, all others are optional.
+
 =head1 BUGS
 
 None so far
